unit WScroll;

{


TWaveScroll

coded by Antonio Bakula (antoniob@usa.net
                         antonio.bakula@zg.hinet.hr)

Donated to public domain. No guaranties of any kind.


TWaveScroll is component that will scroll your text like sine graph.


****************************************************************************

PSEUDOCODE :


    calculate aSin array

    draw text on TextBitmap

    while true do

      copy FXStart colon of TextBitmap to last colon of ScrollBitmap

      Scroll ScrollBitmap

      copy each colon from scrollbitmap from scrollbitmap to screenbitmap,
      and for each colon add sin value to Y start , sin value is from
      aSin array

      Blit screenbitmap to parent DC

      roll aSin array

      increase FXStart

      if FXStart is last colon of TextBitmap then FxStart := 0

    end

****************************************************************************


  aSin array holds values of y coordinate for each colon.

  TextBitmap is bitmap on witch text is drawn.
    Size : x - width of drawn text
           Y - height of font

  ScrollBitmap
    Size : x - Width property of TWaveScroll
           y - height of font

  ScreenBitmap :
    Size : x - Width property of TWaveScroll
           y - height of font * 3



***** JOB SEARCH ****************

I am programmer looking for *REMOTE* job, preferable job would be making
delphi components. I live in Croatia and I don't wont to move, so only option
is remote work, see resume.txt for details.

e-mail:
  antonio.bakula@zg.hinet.hr
  antoniob@usa.net


}


interface

uses
  Windows,Classes,extctrls,graphics,Forms;

type

  TRollValues = 1..8;
  TScrollValues = 1..3;
  TEnum4 = (One, Two, Three, Four);

  {
    Here is dilemma, if I derive my component from TControl I would get
    design time manipulation and all other Tcontrol goodies, but I'll also
    get lot of unnecessary code, at least for this little component, so I decide
    to derive from TComponent and avoid TControl overhead, if you think that
    deriving from TControl would be better, change it :-)
  }
  TWaveScroll = class(TComponent)
  private
    FText: String;
    FTimer: TTimer;
    FEnabled: Boolean;
    FParent: TForm;
    FTimerRes: Cardinal;
    FScrTop: integer;
    FWidth: integer;
    FHeight: integer;
    FScrLeft: integer;
    FFont: TFont;

    // colors
    FBkColor: TColor;
    FFgColor: TColor;

    FDC: HDC; // DC from TForm parent

    // text
    FTextWidth: integer;
    FTextHeight: integer;

    // DC's and bitmaps for scroll, screen and text
    FScrollBitmap: HBitmap; // *** bitmaps are HBitmap not TBitmap
    FScrollDC: HDC;
    FTextBitmap: HBitmap;
    FTextDC: HDC;
    FScreenBitmap: HBitmap;
    FScreenDC: HDC;

    // x start colon
    FXStart: integer;

    // sine array
    aSin: array of integer;

    // roll counter
    FRollCnt: TRollValues;

    // number of pixels scrolled
    FScrollPixels: TScrollValues;

    // number of full sines
    FSinesCount: TEnum4;

    // scroll rect
    FScrollRect: TRect;

    // enabled property handling
    procedure SetEnabled(const Value: Boolean);

    // start and stop, they are called ONLY from SetEnabled
    //                                 ====
    procedure StartScroll;
    procedure StopScroll;

    // calculates aSin array
    procedure CalcFullSine;

    // OnTimer procedure
    procedure TimerFired(Sender: TObject);

    // these are called from timer procedure
    procedure DoDraw;
    procedure RollASinArray;

    // desing time support
    procedure SetParent(const Value: TForm);
    procedure SetTimerRes(const Value: Cardinal);
    procedure SetFont(const Value: TFont);
    procedure SetScrLeft(const Value: integer);
    procedure SetScrTop(const Value: integer);
    procedure SetWidth(const Value: integer);
    procedure SetRollCnt(const Value: TRollValues);
    procedure SetScrollPixels(const Value: TScrollValues);
    procedure SetSinesCount(const Value: TEnum4);
    procedure SetText(const Value: String);

  protected

  public
    // create & destroy
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;

    // notification
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;

    property Parent: TForm read FParent write SetParent;

  published

    // Scrolling text
    property Text: String read FText write SetText;

    property Enabled: Boolean read FEnabled write SetEnabled default false;

    // left and top coordinates and width of drawn text
    property ScrLeft: integer read FScrLeft write SetScrLeft;
    property ScrTop: integer read FScrTop write SetScrTop;
    property Width: integer read FWidth write SetWidth;


    property Font: TFont read FFont write SetFont;
    property TimerRes:Cardinal read FTimerRes write SetTimerRes;

    // colors
    property BkColor: TColor read FBkColor write FBkColor;
    property FgColor: TColor read FFgColor write FFgColor;

    // how many times aSin is rolled
    property RollCnt: TRollValues read FRollCnt write SetRollCnt;
    // how many pixels are scrolled
    property ScrollPixels: TScrollValues read FScrollPixels write SetScrollPixels;
    // number of full sines
    property SinesCount: TEnum4 read FSinesCount write SetSinesCount;

  end;



procedure Register;

implementation


procedure Register;
begin
  RegisterComponents('Samples', [TWaveScroll]);
end;


(******************************************************************************)
(*                                 TWaveScroll                                *)
(******************************************************************************)


// create
constructor TWaveScroll.Create(AOwner: TComponent);
begin
  inherited;

  FEnabled := false;

  // timer
  FTimer := TTimer.Create(self);
  FTimer.Enabled := false;
  FTimer.Interval := 100;
  FTimer.OnTimer := TimerFired;

  FFont := TFont.Create;

  // **PARENT**
  Parent := TForm(AOwner);

  // defaults
  FText := 'TWaveScroll ';

  FScrLeft := 10;
  FScrTop := 10;
  FWidth := 400;

  FRollCnt := 3;
  FScrollPixels := 1;
  FSinesCount := One;

  // FTimer object is created, so calling SetTimerRes is safe
  TimerRes := 20;

  FBkColor := $000080;
  FFgColor := $C0C0C0;

end;


// destroy
destructor TWaveScroll.Destroy;
begin

  Enabled := false;

  FTimer.Free;
  FFont.Free;

  inherited;
end;


// this starts or ends scroll
procedure TWaveScroll.SetEnabled(const Value: Boolean);
begin


  If FEnabled = Value then
    Exit;   // nothing to do

  FEnabled := Value;

  if FEnabled then
    StartScroll
  else
    StopScroll;

end;


{
  TForm parent notifies my component that he will be destroyed, I must stop the
  scroll becouse in TWaveScroll.StartScroll method I get parent window DC.
  StopScroll will release that DC.

  TForm parent handle property is allready destroyed in TWaveScroll.Destroy
  method in this case I will end up with exception in destroy method on

      ReleaseDC(Parent.Handle, FDC);

    this is reason for Notification overriding, becouse here I have valid
    Parent.Handle property.
}
procedure TWaveScroll.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  Enabled := false; // this will release allocated GDI resources, if Enabled
end;


// start scroll
procedure TWaveScroll.StartScroll;
var
  sz: TSize;
begin

  // we don't wont to create another parent GDI window handle,
  // better is to exit (IMHO)
  if not FParent.HandleAllocated then
  begin
    FEnabled := false;
    Exit;
  end;

  // get DC of parent window
  FDC := GetDC(FParent.Handle);

  // create DC's for bitmap
  FScrollDC := CreateCompatibleDC(FDC);
  FTextDC := CreateCompatibleDC(FDC);
  FScreenDC := CreateCompatibleDC(FDC);

  // set font for TextDC
  SelectObject(FTextDC, FFont.Handle);

  // text measurment
  GetTextExtentPoint32(FTextDC, pChar(Text), Length(Text), sz);

  // height and width of text
  FTextHeight := sz.cy;
  FTextWidth := sz.cx;
  // height of screen bitmap
  FHeight := FTextHeight * 3;

  // create *MONOCHROME* bitmaps
  FTextBitmap := CreateBitmap(FTextWidth,FTextHeight, 1, 1, nil);
  FScrollBitmap := CreateBitmap(FWidth, FTextHeight, 1, 1, nil);
  FScreenBitmap := CreateBitmap(FWidth, FHeight, 1, 1, nil);

  // set bitmaps to their DC's
  SelectObject(FTextDC, FTextBitmap);
  SelectObject(FScrollDC, FScrollBitmap);
  SelectObject(FScreenDC, FScreenBitmap);

  // initializing
  PatBlt(FTextDC, 0, 0, FTextWidth, FTextHeight, WHITENESS);
  PatBlt(FScrollDC, 0, 0, FWidth, FTextHeight, WHITENESS);
  PatBlt(FSCreenDC, 0, 0, FWidth, FHeight, WHITENESS);

  // draw text on TextDC
  TextOut(FTextDC,0,0,pChar(Text),Length(Text));

  // scroll rect, it's better to calcuate it here than in time critical DoDraw procedure
  FScrollRect := Rect(0,0,FWidth,FTextHeight);

  // reset start line
  FXStart := 0;

  // calculate values for aSin array
  CalcFullSine;

  // enable timer
  FTimer.Enabled := true;
end;


// scroll stop
procedure TWaveScroll.StopScroll;
begin

  // this DC was obitained with GetDC so i must release it
  ReleaseDC(Parent.Handle, FDC);

  // theese DC's are created
  DeleteDC(FScrollDC);
  DeleteDC(FTextDC);
  DeleteDC(FScreenDC);

  // delete bitmaps
  DeleteObject(FTextBitmap);
  DeleteObject(FScrollBitmap);
  DeleteObject(FScreenBitmap);

  // disable timer
  FTimer.Enabled := false;

  // let parent repaint it's self
  if Parent.HandleAllocated then
    Parent.Repaint;
end;


// calculate values for aSin array
procedure TWaveScroll.CalcFullSine;
var
  step, poc: Extended;
  xx: integer;
begin

  // aSin array, one array element for each colon on screen
  SetLength(aSin, FWidth+1);

  // starting value
  poc := -(3.14 * (Integer(SinesCount)+1));

  // step
  step :=(Abs(poc)*2) / FWidth;

  // fill aSin array
  for xx := 0 to FWidth do
  begin
    poc := poc + step;
    // this is Y value for ScreenDC
    aSin[xx] := FTextHeight + Round(Sin(poc)*FTextHeight);
  end;
end;


// OnTimer
procedure TWaveScroll.TimerFired(Sender: TObject);
var
  xx: integer;
begin

  DoDraw;
  for xx := 1 to RollCnt do
    RollASinArray;

  // increase source colon from FTextBitmap
  Inc(FXStart);
  If FXStart > FTextWidth then
    FXStart := 0;

end;


// draw  routine
procedure TWaveScroll.DoDraw;
var
  cf,cb: LongInt;
  xx:integer;
begin

  // copy Y colon from FTextBitmap to FScrollBitmap
  BitBlt(FScrollDC, FWidth-1, 0, FWidth-1, FTextHeight, FTextDC, FXStart, 0, SRCCOPY);

  // scroll FScrollBitmap
  ScrollDC(FScrollDC, -1* ScrollPixels, 0, FScrollRect, FScrollRect, 0, nil);

  // Clear screen bitmap
  PatBlt(FSCreenDC, 0, 0 , FWidth, FHeight, WHITENESS);

  // copy each colon on FScreenDC
  for xx := 0 to FWidth do
    BitBlt(FScreenDC, xx, aSin[xx], 1,FTextHeight , FScrollDC, xx, 0, SRCCOPY);

  // save colors
  cf := GetTextColor(FDC);
  cb := GetBkColor(FDC);

  // set my colors
  SetTextColor(FDC,FFgColor);
  SetBkColor(FDC,FBkColor);

  // copy screenDC to parent's DC
  BitBlt(FDC, ScrLeft, ScrTop, FWidth, FHeight, FScreenDC, 0, 0, SRCCOPY);

  // restore colors
  SetTextColor(FDC,cf);
  SetBkColor(FDC,cb);

end;


// roll aSin array
procedure TWaveScroll.RollASinArray;
var
  xx, pom: integer;
begin
  pom := aSin[FWidth];
  for xx := FWidth downto 1 do
    aSin[xx] := aSin[xx-1];
   aSin[0] := pom;
end;


(*     DESIGN TIME    *)


procedure TWaveScroll.SetTimerRes(const Value: Cardinal);
begin
  FTimerRes := Value;
  FTimer.Interval := Value;

  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;

end;


procedure TWaveScroll.SetParent(const Value: TForm);
var
  bb: Boolean;
begin

  if Value = FParent then Exit;

  bb := Enabled;

  // release allocated GDI resources for old parent, if any
  Enabled := false;

  // remove old notification from OLD PARENT
  if Assigned(FParent) then
    FParent.RemoveFreeNotification(self);

  // set parent
  FParent := Value;
  // add self in NEW PARENT FreeNotification list
  FParent.FreeNotification(self);

  // if component was enabled
  if bb then
    Enabled := true;

end;



procedure TWaveScroll.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetScrLeft(const Value: integer);
begin
  FScrLeft := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetScrTop(const Value: integer);
begin
  FScrTop := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetWidth(const Value: integer);
begin
  FWidth := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetRollCnt(const Value: TRollValues);
begin
  FRollCnt := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetScrollPixels(const Value: TScrollValues);
begin
  FScrollPixels := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetSinesCount(const Value: TEnum4);
begin
  FSinesCount := Value;
  if Enabled then
  begin
    Enabled := false;
    Enabled := true;
  end;
end;

procedure TWaveScroll.SetText(const Value: String);
begin
  FText := Value;
  if FText[Length(FText)] <> ' ' then
    FText := FText + ' ';
end;

end.
